/*
As its name implies (Cyclic Redundancy Check) a crc adds
redundancywhereas the topic of this group is to remove it. But since
thisquestion comes up often, here is some code (by Rob Warnock
<rpw3@sgi.com>).The following C code does CRC-32 in
BigEndian/BigEndian byte/bit order.That is, the data is sent most
significant byte first, and each of the bitswithin a byte is sent most
significant bit first, as in FDDI. You will needto twiddle with it to
do Ethernet CRC, i.e., BigEndian/LittleEndian byte/bitorder. [Left as
an exercise for the reader.]The CRCs this code generates agree with
the vendor-supplied Verilog modelsof several of the popular FDDI "MAC"
chips.
*/
#include "crc.h"

long crc32(char *buf, int len) { 
  char *p; 
  long crc; 
  if (!crc32_table[1]) /* if not already done, */ 
    init_crc32(); /* build table */ 

  crc = 0xffffffff; /* preload shift register, per CRC-32 spec */

  for (p = buf; len > 0; ++p, --len) 
    crc = (crc << 8) ^ crc32_table[(crc >> 24) ^ *p]; 
  return ~crc; /* transmit complement, per CRC-32 spec */}

/* * Build auxiliary table for parallel byte-at-a-time CRC-32. */

#define CRC32_POLY 0x04c11db7   /* AUTODIN II, Ethernet, & FDDI */

void init_crc32() { 
  int i, j; 
  long c;
  for (i = 0; i < 256; ++i) { 
    for (c = i << 24, j = 8; j > 0; --j) 
      c = c & 0x80000000 ? (c << 1) ^ CRC32_POLY : (c << 1); 
    crc32_table[i] = c;
  }
}





