/*
 * DEBUG: Section 15          dynamic-hierarchy:
 */

/* Hacked from originally from neighbor.c module of SQUID relase ??
 * 
 * Implements dynamically computed per-URL hierarchies using
 * hashing functions.
 *
 * By: Bien Velez-Rivera MIT/LCS
 *
 */

#include "squid.h"

/* BV: Additions to handle dynamic hierarchies */
/* BV: NOTE: In the interest of keeping modifications as localized as
 * BV: possible, I haven't tryed to minimize code duplication.
 */



extern neighbors *myfriends =NULL;

/* Exported functions */

/* dhGetFirstUpParent -
 * Returns the edge structure for the parent that should resolve the next
 * cache miss
 */
edge *dhGetFirstUpParent(request)
     request_t *request;
{
    edge *e = NULL;
    long p=0;
    dhBuildParentSequence(request);
    if (myfriends->n_parent < 1)
	return NULL;
/*     for (e = myfriends->edges_head; e; e = e->next) { */
/* 	if (!e->neighbor_up) */
/* 	    continue; */
/* 	if (e->type != EDGE_PARENT) */
/* 	    continue; */
/* 	if (edgeWouldBePinged(e, request)) */
/* 	    return e; */
/*     } */
    for (p=request->first_upper_parent; p < request->n_parent_next_level; p++)
      {
	e = request->parent_seq[p].parent_edge;
	/*	if (!e->neighbor_up) */
	/*continue; */
/* 	if (edgeWouldBePinged(e, request)) */
/* 	  return e; */
      }
    return NULL;
}

/* Local functions */

/* This is the hashing function that need to be added */
long dhParentHash(parent_url, req_url)
     char *parent_url;
     char *req_url;
{
  long hash_value;

  hash_value = crc32(parent_url,strlen(parent_url)) + crc32(req_url,strlen(req_url));
  return hash_value;
}


void dhBuildParentSequence(request)
     request_t *request;
{
  edge *e = NULL;
  long p = 0;
  parent_t *parent_seq;
  const char *me = getMyHostname();
  parent_hash_t my_hash = dhParentHash(me, request->urlpath);
  long n_parent_upper_levels = 0;
  /*  int  levels = 0; */
  /*  int  branching = 0; */
  /* Only build if hasn't been built before */
  if (request->parent_seq && (myfriends->n_parent > 1)) {
    /* allocate mem for array of parent structures */
    parent_seq = xmalloc(myfriends->n_parent * sizeof(parent_t));
    /* for each parent (edge) in the configuration,
       - calculate hash value
       - set pointer from array entry into edge structure */
    for(e = myfriends->edges_head; e->next; e=e->next) {
      if (e->type == EDGE_PARENT) {
	parent_seq[p].parent_edge = e;
	parent_seq[p].hash_value  = dhParentHash(e->host, request->urlpath);
	p++;
      }
    }
    request->parent_seq = parent_seq;
    /* sort array decreasingly by hash value */
    qsort(parent_seq, myfriends->n_parent, sizeof(parent_t), &dhParentCMP);
    /* must shift sequence past edges before local cache */
    for(p=0; p < myfriends->n_parent; p++) {
      if (parent_seq[p].hash_value > my_hash)
	break;
    }
    request->first_upper_parent = p;
    n_parent_upper_levels = myfriends->n_parent - request->first_upper_parent;
    /* Calculate parents in next level according to hash values */
    request->n_parent_next_level = n_parent_upper_levels; /* Flat hierarchy */
    /* Really
    request->n_parent_next_level = branching ^ (levels - 1)
    Need to get either branching or levels from conf file
    */
  }
}

int dhParentCMP(p1, p2)
     const void *p1;
     const void *p2;
{
  parent_t *parent1 = (parent_t *) p1;
  parent_t *parent2 = (parent_t *) p2;
  if (parent1->hash_value > parent2->hash_value)
    return 1;
  if (parent1->hash_value < parent2->hash_value)
    return -1;
  else
    return 0;
}


int dhParentWouldBePinged(e, request)
     edge *e;
     request_t *request;
{
  dhBuildParentSequence(request);
  /* Decide which parents should be ping'ed */
  /* Alternatives are:
   * - all parents at next level up
   * - the parent with highest hash value in the next level
   * - none of the parents
   */
  return 0; /* No parent should be pinged */
}
